@@ -1,9 +1,9 @@
 ####**********************************************************************
 ####**********************************************************************
 ####
-####  RANDOM SURVIVAL FOREST 3.6.3
+####  RANDOM SURVIVAL FOREST 3.6.4
 ####
-####  Copyright 2009, Cleveland Clinic Foundation
+####  Copyright 2013, Cleveland Clinic Foundation
 ####
 ####  This program is free software; you can redistribute it and/or
 ####  modify it under the terms of the GNU General Public License
@@ -20,73 +20,34 @@
 ####  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ####  Boston, MA  02110-1301, USA.
 ####
-####  ----------------------------------------------------------------
-####  Project Partially Funded By:
-####    --------------------------------------------------------------
-####    National Institutes of Health,  Grant HHSN268200800026C/0001
-####
-####    Michael S. Lauer, M.D., FACC, FAHA 
-####    National Heart, Lung, and Blood Institute
-####    6701 Rockledge Dr, Room 10122
-####    Bethesda, MD 20892
-####
-####    email:  lauerm@nhlbi.nih.gov
-####
-####    --------------------------------------------------------------
-####    Case Western Reserve University/Cleveland Clinic  
-####    CTSA Grant:  UL1 RR024989, National Center for
-####    Research Resources (NCRR), NIH
-####
-####    --------------------------------------------------------------
-####    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
-####    Andy Minn, M.D., Ph.D.
-####    Department of Radiation and Cellular Oncology, and
-####    Ludwig Center for Metastasis Research
-####    The University of Chicago, Jules F. Knapp Center, 
-####    924 East 57th Street, Room R318
-####    Chicago, IL 60637
-#### 
-####    email:  aminn@radonc.uchicago.edu
-####
-####    --------------------------------------------------------------
-####    Bryan Lau, Ph.D.
-####    Department of Medicine, Johns Hopkins School of Medicine,
-####    Baltimore, Maryland 21287
-####
-####    email:  blau1@jhmi.edu
-####
-####  ----------------------------------------------------------------
 ####  Written by:
-####    --------------------------------------------------------------
 ####    Hemant Ishwaran, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
-####    Cleveland Clinic Foundation
-####    9500 Euclid Avenue
-####    Cleveland, OH 44195
+####    Director of Statistical Methodology
+####    Professor, Division of Biostatistics
+####    Clinical Research Building, Room 1058
+####    1120 NW 14th Street
+####    University of Miami, Miami FL 33136
 ####
 ####    email:  hemant.ishwaran@gmail.com
-####    phone:  216-444-9932
-####    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-####
+####    URL:    http://web.ccs.miami.edu/~hishwaran
 ####    --------------------------------------------------------------
 ####    Udaya B. Kogalur, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
+####    Adjunct Staff
+####    Dept of Quantitative Health Sciences
 ####    Cleveland Clinic Foundation
 ####    
-####    Kogalur Shear Corporation
+####    Kogalur & Company, Inc.
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  ubk2101@columbia.edu
-####    phone:  919-824-9825
-####    URL:    www.kogalur-shear.com
+####    email:  commerce@kogalur.com
+####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
 ####**********************************************************************
 ####**********************************************************************
 
 competing.risk <- function (x, plot = TRUE, ...) {
-
   if (sum(inherits(x, c("rsf", "grow"), TRUE) == c(1, 2)) != 2 &
       sum(inherits(x, c("rsf", "predict"), TRUE) == c(1, 2)) != 2)
     stop("This function only works for objects of class `(rsf, grow)' or '(rsf, predict)'.")
@@ -96,8 +57,6 @@ competing.risk <- function (x, plot = TRUE, ...) {
     else {
       rsfPred <- FALSE
   }
-  
-  # number of event types
   if (length(dim(x$ensemble)) == 2) {
     if (rsfPred) {
       ensb <- array(x$ensemble, dim = c(nrow(x$ensemble), ncol(x$ensemble), 1), 1)
@@ -111,11 +70,6 @@ competing.risk <- function (x, plot = TRUE, ...) {
     if (rsfPred) ensb <- x$ensemble else ensb <- x$oob.ensemble 
     n.event <- dim(ensb)[3]
   }
-
-  # ensemble CIF 
-  # ensemble subsurvival functions
-  # ensemble conditional survival functions
-  # ensemble survival function
   surv.ensb.avg <- apply(exp(-ensb[,, 1]), 2, mean, na.rm = TRUE)
   if (n.event > 1) {
     cif.ensb <- sub.ensb <- array(0, dim = c(dim(ensb)[c(1,2)], n.event - 1))
@@ -136,8 +90,6 @@ competing.risk <- function (x, plot = TRUE, ...) {
     dimnames(sub.ensb)[[3]] <- paste("subS.", 1:(n.event-1), sep = "")
     rownames(cond.mort) <- paste("event.", 1:(n.event-1), sep = "")
   }
-    
-  # plots
   matPlot <- function(M, ylab = "Survival (%)", legend = "ensemble survival", pos = 2) {
      m <- dim(cbind(M))[2]
      if (!rsfPred) legend <- paste("oob", legend)     
@@ -152,25 +104,22 @@ competing.risk <- function (x, plot = TRUE, ...) {
   }
   if (plot) {
     if (n.event == 1) {
-      old.par <- par(no.readonly = TRUE)#save par settings
+      old.par <- par(no.readonly = TRUE) 
       matPlot(surv.ensb.avg)
-      par(old.par)#restore par 
+      par(old.par) 
       return(NULL)
     }
     else {
-      old.par <- par(no.readonly = TRUE)#save par settings
+      old.par <- par(no.readonly = TRUE) 
       par(mfrow = c(2,2))
       matPlot(cif.ensb.avg,   "Probability (%)", "ensemble CIF", 1)
       matPlot(sub.ensb.avg,   "Probability (%)", "ensemble subsurvival")
       matPlot(csurv.ensb.avg, "Survival (%)", "ensemble cond. surv.")
       matPlot(surv.ensb.avg)
-      par(old.par)#restore par 
+      par(old.par) 
     }
   }
-    
-  # return the goodies
   if (n.event > 1) {
     invisible(list(cif.ensb = cif.ensb, sub.ensb = sub.ensb, cond.mortality = cond.mort))
   } 
-
 }
