@@ -1,9 +1,9 @@
 ####**********************************************************************
 ####**********************************************************************
 ####
-####  RANDOM SURVIVAL FOREST 3.6.3
+####  RANDOM SURVIVAL FOREST 3.6.4
 ####
-####  Copyright 2009, Cleveland Clinic Foundation
+####  Copyright 2013, Cleveland Clinic Foundation
 ####
 ####  This program is free software; you can redistribute it and/or
 ####  modify it under the terms of the GNU General Public License
@@ -20,74 +20,34 @@
 ####  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ####  Boston, MA  02110-1301, USA.
 ####
-####  ----------------------------------------------------------------
-####  Project Partially Funded By:
-####    --------------------------------------------------------------
-####    National Institutes of Health,  Grant HHSN268200800026C/0001
-####
-####    Michael S. Lauer, M.D., FACC, FAHA 
-####    National Heart, Lung, and Blood Institute
-####    6701 Rockledge Dr, Room 10122
-####    Bethesda, MD 20892
-####
-####    email:  lauerm@nhlbi.nih.gov
-####
-####    --------------------------------------------------------------
-####    Case Western Reserve University/Cleveland Clinic  
-####    CTSA Grant:  UL1 RR024989, National Center for
-####    Research Resources (NCRR), NIH
-####
-####    --------------------------------------------------------------
-####    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
-####    Andy Minn, M.D., Ph.D.
-####    Department of Radiation and Cellular Oncology, and
-####    Ludwig Center for Metastasis Research
-####    The University of Chicago, Jules F. Knapp Center, 
-####    924 East 57th Street, Room R318
-####    Chicago, IL 60637
-#### 
-####    email:  aminn@radonc.uchicago.edu
-####
-####    --------------------------------------------------------------
-####    Bryan Lau, Ph.D.
-####    Department of Medicine, Johns Hopkins School of Medicine,
-####    Baltimore, Maryland 21287
-####
-####    email:  blau1@jhmi.edu
-####
-####  ----------------------------------------------------------------
 ####  Written by:
-####    --------------------------------------------------------------
 ####    Hemant Ishwaran, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
-####    Cleveland Clinic Foundation
-####    9500 Euclid Avenue
-####    Cleveland, OH 44195
+####    Director of Statistical Methodology
+####    Professor, Division of Biostatistics
+####    Clinical Research Building, Room 1058
+####    1120 NW 14th Street
+####    University of Miami, Miami FL 33136
 ####
 ####    email:  hemant.ishwaran@gmail.com
-####    phone:  216-444-9932
-####    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-####
+####    URL:    http://web.ccs.miami.edu/~hishwaran
 ####    --------------------------------------------------------------
 ####    Udaya B. Kogalur, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
+####    Adjunct Staff
+####    Dept of Quantitative Health Sciences
 ####    Cleveland Clinic Foundation
 ####    
-####    Kogalur Shear Corporation
+####    Kogalur & Company, Inc.
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  ubk2101@columbia.edu
-####    phone:  919-824-9825
-####    URL:    www.kogalur-shear.com
+####    email:  commerce@kogalur.com
+####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
 ####**********************************************************************
 ####**********************************************************************
 
-plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
-
-    ### check that object is interpretable
+plot.ensemble.rsf <- function (x, plots.one.page = TRUE, ...) {
     if (sum(inherits(x, c("rsf", "grow"), TRUE) == c(1, 2)) != 2 &
       sum(inherits(x, c("rsf", "predict"), TRUE) == c(1, 2)) != 2)
       stop("This function only works for objects of class `(rsf, grow)' or '(rsf, predict)'.")
@@ -97,21 +57,12 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
     else {
       rsfPred <- FALSE
     }
-    
-    ### null case can occur for '(rsf, predict)' objects, so check 
     if (is.null(x$ndead)) return()
-
-    ### use imputed missing time or censoring indicators
     if (!is.null(x$imputedIndv)) {
       x$cens[x$imputedIndv]=x$imputedData[,1]
       x$time[x$imputedIndv]=x$imputedData[,2]
     }
-    
-    ### no point in producing plots if sample size too small or
-    ### not enough deaths
     if (x$n < 5 | sum(x$cens) < 2) return()
-
-    # use OOB values for grow forest
     if (rsfPred) {
       mort    <- x$mortality
       ensb    <- x$ensemble
@@ -126,16 +77,11 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
       title.1 <- "OOB Ensemble Survival"
       title.2 <- "OOB Mortality vs Time"
     }
-
-    # number of event types
     n.event  <- length(unique(na.omit(x$cens)[na.omit(x$cens) > 0]))
     if (n.event > 1) ensb <- ensb[,,1]
-
-    # survival curves
     surv.ensb <- t(exp(-ensb))
     surv.mean.ensb <- apply(surv.ensb, 1, mean, na.rm = TRUE)
     if (!rsfPred) {
-      ## KM estimator for survival distributin
       Y <- sapply(1:length(x$timeInterest),
                  function(j, tau, t.unq) {sum(tau >= t.unq[j])},
                  tau = x$time,
@@ -146,9 +92,7 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
                  t.unq = x$timeInterest)
       r <- d/(Y+1*(Y == 0))
       surv.aalen <- exp(-cumsum(r))
-
       sIndex <- function(Ju,Ev) { sapply(1:length(Ev), function(j) {sum(Ju <= Ev[j])}) }
-      ## KM estimator for censoring distribution
       censTime <- sort(unique(x$time[x$cens == 0]))
       censTime.pt <- sIndex(censTime, x$timeInterest)
       Y <- sapply(1:length(censTime),
@@ -162,8 +106,6 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
       r <- d/(Y+1*(Y == 0))
       cens.aalen <- c(1, exp(-cumsum(r)))[1+censTime.pt]
     }
-
-    # Brier score stratified by mortality percentiles
     if (!rsfPred) {
       brier.wt <- t(apply(cbind(1:x$n),
                    1,
@@ -190,8 +132,6 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
       brier.score <- cbind(brier.score, sapply(1:length(x$timeInterest),
                                   function(j) {mean(brier.wt[, j]*delta[, j], na.rm = TRUE)}))
     }
-    
-    # plots
     old.par <- par(no.readonly = TRUE)
     if (plots.one.page) {
       if (!rsfPred) par(mfrow = c(2,2)) else par(mfrow = c(1,2))
@@ -269,3 +209,4 @@ plot.ensemble <- function (x, plots.one.page = TRUE, ...) {
     rug(x$timeInterest, ticksize=-0.03)
     par(old.par)      
 }
+plot.ensemble <- plot.ensemble.rsf
