@@ -1,9 +1,9 @@
 ####**********************************************************************
 ####**********************************************************************
 ####
-####  RANDOM SURVIVAL FOREST 3.6.3
+####  RANDOM SURVIVAL FOREST 3.6.4
 ####
-####  Copyright 2009, Cleveland Clinic Foundation
+####  Copyright 2013, Cleveland Clinic Foundation
 ####
 ####  This program is free software; you can redistribute it and/or
 ####  modify it under the terms of the GNU General Public License
@@ -20,79 +20,33 @@
 ####  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ####  Boston, MA  02110-1301, USA.
 ####
-####  ----------------------------------------------------------------
-####  Project Partially Funded By:
-####    --------------------------------------------------------------
-####    National Institutes of Health,  Grant HHSN268200800026C/0001
-####
-####    Michael S. Lauer, M.D., FACC, FAHA 
-####    National Heart, Lung, and Blood Institute
-####    6701 Rockledge Dr, Room 10122
-####    Bethesda, MD 20892
-####
-####    email:  lauerm@nhlbi.nih.gov
-####
-####    --------------------------------------------------------------
-####    Case Western Reserve University/Cleveland Clinic  
-####    CTSA Grant:  UL1 RR024989, National Center for
-####    Research Resources (NCRR), NIH
-####
-####    --------------------------------------------------------------
-####    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
-####    Andy Minn, M.D., Ph.D.
-####    Department of Radiation and Cellular Oncology, and
-####    Ludwig Center for Metastasis Research
-####    The University of Chicago, Jules F. Knapp Center, 
-####    924 East 57th Street, Room R318
-####    Chicago, IL 60637
-#### 
-####    email:  aminn@radonc.uchicago.edu
-####
-####    --------------------------------------------------------------
-####    Bryan Lau, Ph.D.
-####    Department of Medicine, Johns Hopkins School of Medicine,
-####    Baltimore, Maryland 21287
-####
-####    email:  blau1@jhmi.edu
-####
-####  ----------------------------------------------------------------
 ####  Written by:
-####    --------------------------------------------------------------
 ####    Hemant Ishwaran, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
-####    Cleveland Clinic Foundation
-####    9500 Euclid Avenue
-####    Cleveland, OH 44195
+####    Director of Statistical Methodology
+####    Professor, Division of Biostatistics
+####    Clinical Research Building, Room 1058
+####    1120 NW 14th Street
+####    University of Miami, Miami FL 33136
 ####
 ####    email:  hemant.ishwaran@gmail.com
-####    phone:  216-444-9932
-####    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-####
+####    URL:    http://web.ccs.miami.edu/~hishwaran
 ####    --------------------------------------------------------------
 ####    Udaya B. Kogalur, Ph.D.
-####    Dept of Quantitative Health Sciences/Wb4
+####    Adjunct Staff
+####    Dept of Quantitative Health Sciences
 ####    Cleveland Clinic Foundation
 ####    
-####    Kogalur Shear Corporation
+####    Kogalur & Company, Inc.
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  ubk2101@columbia.edu
-####    phone:  919-824-9825
-####    URL:    www.kogalur-shear.com
+####    email:  commerce@kogalur.com
+####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
 ####**********************************************************************
 ####**********************************************************************
 
-########################################################################
-# Primary R function for Random Surival Forests
-# ----------------------------------------------------------------------
-#  Prediction and variable selection for right censored survival and
-#  competing risk data using Random Survival Forests (RSF) (Ishwaran,
-#  Kogalur, Blackstone and Lauer, 2008).
-########################################################################
-
 rsf <- function(
     formula,#
     data = NULL,#
@@ -114,8 +68,7 @@ rsf <- function(
     do.trace = FALSE,#
     ...)
 {
-
-  ## Preliminary checks for formula and data
+  warning("\nThis package has been deprecated. Please upgrade to the new package 'randomForestSRC'.\n")
   if (!inherits(formula, "formula")) stop("'formula' is not a formula object.")
   if (is.null(data)) stop("'data' is missing.")
   if (!is.data.frame(data)) stop("'data' must be a data frame.")
@@ -134,16 +87,8 @@ rsf <- function(
   else {
     impute.only <- FALSE
   }
-
-  ## Ensure backward compatability with 3.0 user scripts
   if (importance == TRUE)  importance <- "permute"
   if (importance == FALSE) importance <- "none"    
-
-  ## Preliminary processing of missing data:
-  ## If all survival times or all censoring values missing: stop
-  ## Remove all predictors will missing values in all entries
-  ## Next remove all records with missing values in all entries
-  ## At each step check if enough data remains
   pt.Time <- is.na(data[,is.element(names(data), fNames[1])])
   pt.Cens <- is.na(data[,is.element(names(data), fNames[2])])
   if (all(pt.Time) | all(pt.Cens))
@@ -156,11 +101,6 @@ rsf <- function(
   data <- data[!pt.row, ]    
   if (ncol(data) <= 1)
     stop("Less than 2 records in the NA-processed data.  Analysis not meaningful.")
-
-  ## Get predictor names and formula
-  ## If factor has an NA in its levels, remove it
-  ## Identify unordered/ordered factors 
-  ## Save factor levels for immutable map
   if (fNames[3] != ".") {
     if (!big.data) {
       predTempNames <- attr(terms(formula), "term.labels")
@@ -182,10 +122,6 @@ rsf <- function(
   xfactor.levels <- get.factor$xfactor.levels
   xfactor.order <- get.factor$xfactor.order
   xfactor.order.levels <- get.factor$xfactor.order.levels
-
-  ## Data conversion to numeric mode
-  ## Get predictors
-  ## Be mindful of NA's
   if (na.action != "na.omit" & na.action != "na.impute") na.action <- "na.omit"
   if (!big.data) {
     data <- as.data.frame(data.matrix(data))
@@ -197,7 +133,7 @@ rsf <- function(
                  paste(c(fNames[1:2], predTempNames), collapse="+"))), data))
     options(na.action=old.na.action)
   }
-  else {#big.data
+  else { 
     data <- data[ , is.element(names(data), c(fNames[1:2], predTempNames))]
     data <- as.data.frame(data.matrix(data))
     if (na.action == "na.omit") {
@@ -211,8 +147,6 @@ rsf <- function(
   ncov <- ncol(predictors)
   if (n <= 1)
     stop("Less than two records in the NA-processed data.  Analysis not meaningful.")
-
-  ## Set predictor type
   predictorType <- rep("R", ncov)
   if (length(xfactor) > 0) {
     predictorType[is.element(predictorNames, xfactor)] <- "C"
@@ -220,11 +154,8 @@ rsf <- function(
   if (length(xfactor.order) > 0) {
     predictorType[is.element(predictorNames, xfactor.order)] <- "I"
   }
-  ## Set predictor weight
-  ## Check that nsplit is set correctly 
   nsplit <- round(nsplit)
   if (nsplit < 0) stop("Invalid nsplit value specified.")    
-
   if (is.null(predictorWt)) {
     predictorWt <- rep(1/ncov, ncov)
   }
@@ -236,14 +167,10 @@ rsf <- function(
       predictorWt <-predictorWt/sum(predictorWt)
     }
   }
-
-  ## Final checks on option parameters
   ntree <- round(ntree)
   if (ntree < 1) stop("Invalid choice of 'ntree'.  Cannot be less than 1.")
-
   nimpute <- round(nimpute)
   if (nimpute < 1) stop("Invalid choice of 'nimpute'.  Cannot be less than 1.")
-
   if (!is.null(mtry)) {
     mtry <- round(mtry)
     if (mtry < 1 | mtry > ncov) mtry <- max(1, min(mtry, ncov))
@@ -251,39 +178,21 @@ rsf <- function(
   else {
     mtry <- max(floor(sqrt(ncov)), 1)
   }
-
   if (is.null(seed) || abs(seed)<1) seed <- runif(1,1,1e6)
   seed <- -round(abs(seed))
-
-  ## Time and censoring checks:
-  ## Check to see if there are deaths.  
-  ## Check that censoring and time are properly coded.
   Time <- data[,is.element(names(data), fNames[1])]
   Cens <- data[,is.element(names(data), fNames[2])]
   nMiss <- sum(is.na(Cens) | is.na(Time) | apply(predictors, 1, function(x){any(is.na(x))}))
   if (all(na.omit(Cens) == 0)) {
     stop("No deaths in data!")
   }
-
-  ## Extract the unique event types.
   eventType <- unique(na.omit(Cens))
-  ## Ensure they are all greater than or equal to zero.
   if (sum(eventType >= 0) != length(eventType)) {
     stop("Censoring variable must be coded as NA, 0, or greater than 0.")    
   }
-  ## Discard the censored state, if it exists.
   eventType <- eventType[eventType > 0]
-
-  
-  ## Ensure coherency of the split rule.  If it has not been specified, and
-  ## the data set does not contain competing risks (CR), then the split rule is
-  ## logrank.  If it has not been specified, and the data set does contain CR, then
-  ## the split rule is logrankCR.  This allows the user to specify a non-CR analysis
-  ## even if the data contains CR.
-  
   splitrule.names <- c("logrank", "conserve", "logrankscore", "random", "logrankCR")
   if (is.null(splitrule)) {
-    ## No split rule specified
     if (length(eventType) ==  1) {
       splitrule.idx <- which(splitrule.names == "logrank")
     }
@@ -293,33 +202,21 @@ rsf <- function(
     splitrule <- splitrule.names[splitrule.idx]
   }
   else {
-    ## User specified split rule
     splitrule.idx <- which(splitrule.names == splitrule)
     if (length(splitrule.idx) != 1) {
       stop("Invalid split rule specified:  ", splitrule)
     if (length(eventType) ==  1 & splitrule.idx == 5)
       stop("Cannot specify logrankCR splitting for right-censored data")
     }
-    ## the native code allows nsplit > 0, but is dynamically resized
     if (splitrule == "random" & nsplit == 0) {
       nsplit <- 1
     }
   }
-
-  ## Initialize nsplit, noting that it may be been overridden.
   splitrule.nsplit <- nsplit
-
-
   if (!all(na.omit(Time) > 0)) {
     stop("Time must be strictly positive.")
   }
-
-  ## Don't need the data anymore
   remove(data)
-  
-  ## Set grid of time points
-  ## Final checks for event time consistency
-  ## Set nodesize
   nonMissingOutcome <- which(!is.na(Cens) & !is.na(Time))
   nonMissingDeathFlag <- (Cens[nonMissingOutcome] != 0)    
   timeInterest <- sort(unique(Time[nonMissingOutcome[nonMissingDeathFlag]]))
@@ -332,9 +229,6 @@ rsf <- function(
   else {
     nodesize <- min(c(round(0.632*sum(na.omit(Cens) != 0)), c(3, 6)[1+1*(length(eventType) > 1)], N))
   }
-
-  ## Check status of impute mode (internal use only).  Check all is ok.
-  ## Otherwise ... work out individuals at risk (used later for mortality calculation)
   if (impute.only) {
     if (nMiss == 0) stop("data has no missing values, using 'impute' makes no sense")
   }
@@ -346,8 +240,6 @@ rsf <- function(
     Risk <- Risk - c(Risk[-1] , 0)
     impute.only <- FALSE
   }   
-
-  ## Convert trace into native code parameter
   if (!is.logical(do.trace)) {
     if (do.trace >= 1) {
       do.trace <- 2^24 * round(do.trace) + 1
@@ -359,8 +251,6 @@ rsf <- function(
   else {
     do.trace <- 1 * do.trace
   }
-
-  ## Convert importance option into native code parameter
   if (importance == "none") {
     importanceBits <- 0
   }
@@ -373,9 +263,6 @@ rsf <- function(
   else {
     stop("Invalid choice for 'importance' option:  ", importance)
   }
-
-
-  ## Convert varUsed option into native code parameter
   if (is.null(varUsed)) {
     varUsedBits <- 0
   }
@@ -388,89 +275,12 @@ rsf <- function(
   else {
     stop("Invalid choice for 'varUsed' option:  ", varUsed)
   }
-
-  ## Convert impute.only option into native code parameter.
-  ## It will always be initialized.
   if (impute.only == TRUE) {
     impute.only <- 2^16
   }
   else {
     impute.only <- 0
   }
-
-####################################################################
-  ## rsf.default(...)
-  ##
-  ## Parameters passed:
-  ## #################################################################
-  ## 00 - C function call
-  ##
-  ## 01 - trace output flag
-  ##    -  0 = no trace output
-  ##    - !0 = various levels of trace output
-  ##
-  ## 02 - option protocol for output objects.
-  ##    - note that the native code is capable of some optional
-  ##      outputs not implemented on the R-side by (**).
-  ##
-  ##      GROW PRED INTR IMPUTE
-  ##       *         *          0x000001 = OENS
-  ##       *    *               0x000002 = FENS
-  ##       *    *    *          0x000004 = PERF
-  ##       **   **              0x000008 = PROX
-  ##       *    *    *    (**)  0x000010 = LEAF
-  ##       **                   0x000020 = TREE \ part of 
-  ##       **                   0x000040 = SEED / the forest 
-  ##       **   **   **    *    0x000080 = MISS
-  ##       ***       **    *    0x000100 = OMIS
-  ##       **   **   *          0x000200 = \  VIMP_TYPE
-  ##       **   **   *          0x000400 =  | VIMP_JOIN
-  ##       **   **   *          0x000800 = /  VIMP
-  ##       **  (**) (**)        0x001000 = \  VUSE_TYPE
-  ##       **  (**) (**)        0x002000 = /  VUSE
-  ##      (**) (**)  *          0x004000 = POUT_TYPE (non SEXP)
-  ##       **   **   (**)       0x008000 = SPLT_DPTH
-  ##       *         *                     OPOE (tracks OENS)
-  ##       *    *                          FPOE (tracks FENS)
-  ##
-  ##       *   default  output
-  ##       **  optional output
-  ##      (**) optional output not implemented on the R-side
-  ##
-  ## 03 - random seed for repeatability, integer < 0 
-  ## 04 - split rule to be used 
-  ##       1 = Log-Rank
-  ##       2 = Conservation of Events
-  ##       3 = Log-Rank Score
-  ##       4 = Random Split
-  ##       5 = Log-Rank CR
-  ## 05 - number of random split points, non-negative integer
-  ## 06 - number of covariates to be randomly selected for
-  ##      growing tree, integer > 0
-  ## 07 - number of trees in the forest, integer > 0
-  ## 08 - minimum number of deaths allowed in a node, integer > 0
-  ## 09 - number of observations in data set, integer > 1
-  ## 10 - vector of observed event times, doubles > 0
-  ## 11 - vector of observed event types
-  ##         0  = censored
-  ##       > 0  = events
-  ## 12 - number of predictors, integer > 0
-  ## 13 - [p x n] matrix of predictor observations
-  ## 14 - number of time points of interest, integer > 0
-  ## 15 - vector of time points of interest, doubles
-  ## 16 - vector of random covariate weights, doubles
-  ## 17 - vector of predictor types
-  ##       "R" - real value
-  ##       "I" - integer value 
-  ##       "C" - categorical 
-  ## 18 - number of impute iterations, integer > 0
-  ## 
-  ## #################################################################
-
-  ## ###########################################################
-  ##  SEXP outputs:  See parameter 2 above and note "*".
-  ## ###########################################################    
-
   nativeOutput <- .Call("rsfGrow",
                         as.integer(do.trace),
                         as.integer(impute.only +
@@ -495,26 +305,16 @@ rsf <- function(
                         as.double(predictorWt),
                         as.character(predictorType),
                         as.integer(nimpute))
-
-  ## check for error return condition in the native code
   if(is.null(nativeOutput)) {
     stop("Error occurred in algorithm.  Please turn trace on for further analysis.")
   }
-    
-  ## Check if there was missing data, and assign imputed data if so.
   if (nMiss > 0) {
     imputedData <- matrix(nativeOutput$imputation, nrow = nMiss, byrow = FALSE)
     imputedIndv <- imputedData[,1]
     imputedData <- as.matrix(imputedData[,-1])
     if (nMiss == 1) imputedData <- t(imputedData)
-    
-    ## Fill NA's in original GROW data with multiply imputed values.
-    ## This will now serve as the forest data set and will enable
-    ## recovery of terminal node membership with the head of the
-    ## seed chain.
     if (nimpute > 1) {
       if (nimpute == 2) {
-        ## The forest was grown using overlaid OOB summary values.
         imputedOOBData <- matrix(nativeOutput$oobImputation, nrow = nMiss, byrow = FALSE)
         imputedOOBData <- as.matrix(imputedOOBData[,-1])
         if (nMiss == 1) imputedOOBData <- t(imputedOOBData)
@@ -523,32 +323,21 @@ rsf <- function(
         predictors[imputedIndv, ] <- imputedOOBData[, -1:-2]
       }
       else {
-        ## The forest was grown using overlaid full (all) summary values.         
         Cens[imputedIndv] <- imputedData[, 1]
         Time[imputedIndv] <- imputedData[, 2]
         predictors[imputedIndv, ] <- imputedData[, -1:-2]
       }
-      ## Remove the imputed data outputs.
       imputedIndv    <- NULL
       imputedData    <- NULL
       imputedOOBData <- NULL
-      
     }  
     else {
-      ## Add column names to the imputed data outputs in the absence
-      ## of multiple imputation.
-      ## names(Cens) = fNames[2]
-      ## names(Time) = fNames[1]    
       colnames(imputedData) <- c(fNames[2], fNames[1], predictorNames)
       imputedData=as.data.frame(imputedData)
     }
   }
-
-  ## Add column names to predictor matrix 
   predictors <- as.data.frame(predictors)
   colnames(predictors) <- predictorNames
-
-  ## Map predictor factors back to original values
   if (length(xfactor) > 0) {
     for (k in 1:length(xfactor)) {      
       ptk <- (colnames(predictors) == xfactor[k])
@@ -575,8 +364,6 @@ rsf <- function(
       predictors[ , ptk] <- xk
     }
   }
-  
-  ## Map imputed data factors back to original values
   if (length(xfactor) > 0 & nMiss > 0 & nimpute < 2) {
     for (k in 1:length(xfactor)) {
       ptk <- (colnames(imputedData) == xfactor[k])
@@ -603,9 +390,6 @@ rsf <- function(
       imputedData[ , ptk] <- xk
     }
   }
-
-  ## Get varUsed information
-  ## Add names for nicety
   if (!is.null(varUsed)) {
     if (varUsed == "all.trees") {
       varUsed <- nativeOutput$varUsed
@@ -616,8 +400,6 @@ rsf <- function(
       colnames(varUsed) <- predictorNames
     }
   }
-
-  ## Define the forest.
   if (forest) {
     nativeArray <- as.data.frame(cbind(nativeOutput$treeID,
                                        nativeOutput$nodeID,
@@ -625,10 +407,7 @@ rsf <- function(
                                        nativeOutput$contPT,
                                        nativeOutput$mwcpSZ))
     names(nativeArray) <- c("treeID", "nodeID", "parmID", "contPT", "mwcpSZ")
-
-    ## This can be NULL if there are no factor splits.
     nativeFactorArray <- nativeOutput$mwcpPT
-    
     forest <- list(nativeArray = nativeArray,
                    nativeFactorArray = nativeFactorArray,
                    timeInterest = timeInterest, 
@@ -643,11 +422,7 @@ rsf <- function(
   else {
     forest <- NULL
   }
-
-  # number of event types
-  # pretty names
   if ((length(eventType) > 1) & (splitrule.idx >= 4)) {
-    ## Split rule can be logrankCR or random
     n.event <- length(eventType) + 1
     ens.names <- list(NULL, NULL, c("CHF", paste("condCHF.", 1:(n.event-1), sep = "")))
     err.names <- list(c("CHF", paste("condCHF.", 1:(n.event - 1), sep = "")), NULL)
@@ -655,23 +430,18 @@ rsf <- function(
     poe.names <- list(paste("event.", 1:(n.event - 1), sep = ""), NULL)
   }
   else {
-    # map all non-zero, non-na censoring values to a unique integer value
     Cens[Cens != 0 & !is.na(Cens)] <-  min(Cens[Cens != 0 & !is.na(Cens)])
     n.event <- 1
     ens.names <- list(NULL, NULL, NULL)
     vimp.names <- list(NULL, predictorNames)
     err.names <- list(NULL, NULL)
   }
-
-  ## ensemble mortality (in-bag and OOB)
   if (!impute.only) {
     mortality <- apply(array(nativeOutput$fullEnsemble, c(n, length(timeInterest), n.event))[,,1],
                      1, function(x, wt) {sum(x*wt)}, wt = Risk)
     oob.mortality <- apply(array(nativeOutput$oobEnsemble, c(n, length(timeInterest), n.event))[,,1],
                      1,function(x, wt) {sum(x*wt)}, wt = Risk)
   }
-  
-  ## VIMP
   make.vec <- function(x) {
     if (!is.null(dim(x)) && nrow(x) == 1) {
       x.names <- colnames(x)
@@ -692,16 +462,12 @@ rsf <- function(
   else {
     VIMP <- NULL
   }
-
-  ## assign NULL values under impute mode (some redundancy)
   if (impute.only) {
     nativeOutput$fullEnsemble <- nativeOutput$oobEnsemble <- 
     nativeOutput$fullPOE <- nativeOutput$oobPOE <- mortality <- oob.mortality <-
     nativeOutput$performance <- nativeOutput$leafCount <- VIMP <- forest <-
     nativeOutput$proximity <- varUsed <-  NULL
   }
-
-  ## make the output object
   rsfOutput <- list(
     call = match.call(),
     formula = formula,
@@ -743,15 +509,11 @@ rsf <- function(
     imputedData = (if (nMiss > 0) imputedData else NULL),
     splitDepth  = (if (split.depth) matrix(nativeOutput$splitDepth, nrow=n, byrow=FALSE) else NULL)
   )
-
   if (!big.data) {
     class(rsfOutput) <- c("rsf", "grow")
   }
   else {
     class(rsfOutput) <- c("rsf", "grow", "bigdata")
   }
-
   return(rsfOutput)
-  
 }
-
