@@ -1,9 +1,9 @@
 ////**********************************************************************
 ////**********************************************************************
 ////
-////  RANDOM SURVIVAL FOREST 3.6.3
+////  RANDOM SURVIVAL FOREST 3.6.4
 ////
-////  Copyright 2009, Cleveland Clinic Foundation
+////  Copyright 2013, Cleveland Clinic Foundation
 ////
 ////  This program is free software; you can redistribute it and/or
 ////  modify it under the terms of the GNU General Public License
@@ -20,66 +20,28 @@
 ////  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ////  Boston, MA  02110-1301, USA.
 ////
-////  ----------------------------------------------------------------
-////  Project Partially Funded By:
-////    --------------------------------------------------------------
-////    National Institutes of Health,  Grant HHSN268200800026C/0001
-////
-////    Michael S. Lauer, M.D., FACC, FAHA 
-////    National Heart, Lung, and Blood Institute
-////    6701 Rockledge Dr, Room 10122
-////    Bethesda, MD 20892
-////
-////    email:  lauerm@nhlbi.nih.gov
-////
-////    --------------------------------------------------------------
-////    Case Western Reserve University/Cleveland Clinic  
-////    CTSA Grant:  UL1 RR024989, National Center for
-////    Research Resources (NCRR), NIH
-////
-////    --------------------------------------------------------------
-////    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
-////    Andy Minn, M.D., Ph.D.
-////    Department of Radiation and Cellular Oncology, and
-////    Ludwig Center for Metastasis Research
-////    The University of Chicago, Jules F. Knapp Center, 
-////    924 East 57th Street, Room R318
-////    Chicago, IL 60637
-//// 
-////    email:  aminn@radonc.uchicago.edu
-////
-////    --------------------------------------------------------------
-////    Bryan Lau, Ph.D.
-////    Department of Medicine, Johns Hopkins School of Medicine,
-////    Baltimore, Maryland 21287
-////
-////    email:  blau1@jhmi.edu
-////
-////  ----------------------------------------------------------------
 ////  Written by:
-////    --------------------------------------------------------------
 ////    Hemant Ishwaran, Ph.D.
-////    Dept of Quantitative Health Sciences/Wb4
-////    Cleveland Clinic Foundation
-////    9500 Euclid Avenue
-////    Cleveland, OH 44195
+////    Director of Statistical Methodology
+////    Professor, Division of Biostatistics
+////    Clinical Research Building, Room 1058
+////    1120 NW 14th Street
+////    University of Miami, Miami FL 33136
 ////
 ////    email:  hemant.ishwaran@gmail.com
-////    phone:  216-444-9932
-////    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-////
+////    URL:    http://web.ccs.miami.edu/~hishwaran
 ////    --------------------------------------------------------------
 ////    Udaya B. Kogalur, Ph.D.
-////    Dept of Quantitative Health Sciences/Wb4
+////    Adjunct Staff
+////    Dept of Quantitative Health Sciences
 ////    Cleveland Clinic Foundation
 ////    
-////    Kogalur Shear Corporation
+////    Kogalur & Company, Inc.
 ////    5425 Nestleway Drive, Suite L1
 ////    Clemmons, NC 27012
 ////
-////    email:  ubk2101@columbia.edu
-////    phone:  919-824-9825
-////    URL:    www.kogalur-shear.com
+////    email:  commerce@kogalur.com
+////    URL:    http://www.kogalur.com
 ////    --------------------------------------------------------------
 ////
 ////**********************************************************************
@@ -271,7 +233,6 @@ SEXP rsf(char mode, uint traceFlag) {
   Node  *rootPtr;
   Node  *terminalNode;
   uint   rejectedTreeCount;  
-  uint   obsSize;
   double  *statusPtr;
   double  *timePtr;
   double **predictorPtr;
@@ -279,7 +240,6 @@ SEXP rsf(char mode, uint traceFlag) {
   uint  *mwcpPtr;
   char  result;
   uint i, j, k, b, r;
-  clock_t totalTime, startTime;
   statusPtr    = NULL;  
   timePtr      = NULL;  
   predictorPtr = NULL;  
@@ -296,7 +256,6 @@ SEXP rsf(char mode, uint traceFlag) {
   _cindxTime   = 0;
   setTraceFlag(traceFlag);
   updateTraceFlag(TRUE);
-  startTime = clock();
   if (_imputeSize < 1) {
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Parameter verification failed.");
@@ -340,8 +299,7 @@ SEXP rsf(char mode, uint traceFlag) {
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Unknown case in switch encountered. ");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
     break;
   }
   initializeArrays(mode);
@@ -832,8 +790,7 @@ SEXP rsf(char mode, uint traceFlag) {
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Unknown case in switch encountered. ");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
     break;
   }
   unstackPreDefinedCommonArrays();
