@@ -1,9 +1,9 @@
 ////**********************************************************************
 ////**********************************************************************
 ////
-////  RANDOM SURVIVAL FOREST 3.6.3
+////  RANDOM SURVIVAL FOREST 3.6.4
 ////
-////  Copyright 2009, Cleveland Clinic Foundation
+////  Copyright 2013, Cleveland Clinic Foundation
 ////
 ////  This program is free software; you can redistribute it and/or
 ////  modify it under the terms of the GNU General Public License
@@ -20,66 +20,28 @@
 ////  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ////  Boston, MA  02110-1301, USA.
 ////
-////  ----------------------------------------------------------------
-////  Project Partially Funded By:
-////    --------------------------------------------------------------
-////    National Institutes of Health,  Grant HHSN268200800026C/0001
-////
-////    Michael S. Lauer, M.D., FACC, FAHA 
-////    National Heart, Lung, and Blood Institute
-////    6701 Rockledge Dr, Room 10122
-////    Bethesda, MD 20892
-////
-////    email:  lauerm@nhlbi.nih.gov
-////
-////    --------------------------------------------------------------
-////    Case Western Reserve University/Cleveland Clinic  
-////    CTSA Grant:  UL1 RR024989, National Center for
-////    Research Resources (NCRR), NIH
-////
-////    --------------------------------------------------------------
-////    Dept of Defense Era of Hope Scholar Award, Grant W81XWH0910339
-////    Andy Minn, M.D., Ph.D.
-////    Department of Radiation and Cellular Oncology, and
-////    Ludwig Center for Metastasis Research
-////    The University of Chicago, Jules F. Knapp Center, 
-////    924 East 57th Street, Room R318
-////    Chicago, IL 60637
-//// 
-////    email:  aminn@radonc.uchicago.edu
-////
-////    --------------------------------------------------------------
-////    Bryan Lau, Ph.D.
-////    Department of Medicine, Johns Hopkins School of Medicine,
-////    Baltimore, Maryland 21287
-////
-////    email:  blau1@jhmi.edu
-////
-////  ----------------------------------------------------------------
 ////  Written by:
-////    --------------------------------------------------------------
 ////    Hemant Ishwaran, Ph.D.
-////    Dept of Quantitative Health Sciences/Wb4
-////    Cleveland Clinic Foundation
-////    9500 Euclid Avenue
-////    Cleveland, OH 44195
+////    Director of Statistical Methodology
+////    Professor, Division of Biostatistics
+////    Clinical Research Building, Room 1058
+////    1120 NW 14th Street
+////    University of Miami, Miami FL 33136
 ////
 ////    email:  hemant.ishwaran@gmail.com
-////    phone:  216-444-9932
-////    URL:    www.bio.ri.ccf.org/Resume/Pages/Ishwaran/ishwaran.html
-////
+////    URL:    http://web.ccs.miami.edu/~hishwaran
 ////    --------------------------------------------------------------
 ////    Udaya B. Kogalur, Ph.D.
-////    Dept of Quantitative Health Sciences/Wb4
+////    Adjunct Staff
+////    Dept of Quantitative Health Sciences
 ////    Cleveland Clinic Foundation
 ////    
-////    Kogalur Shear Corporation
+////    Kogalur & Company, Inc.
 ////    5425 Nestleway Drive, Suite L1
 ////    Clemmons, NC 27012
 ////
-////    email:  ubk2101@columbia.edu
-////    phone:  919-824-9825
-////    URL:    www.kogalur-shear.com
+////    email:  commerce@kogalur.com
+////    URL:    http://www.kogalur.com
 ////    --------------------------------------------------------------
 ////
 ////**********************************************************************
@@ -219,8 +181,7 @@ void initializeFactorArrays(char mode) {
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Attempt to initialize factorness in its absence.");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
   }
   _maxFactorLevel = 0;
   for (j = 1; j <= _factorCount; j++) {
@@ -236,8 +197,7 @@ void initializeFactorArrays(char mode) {
         else {
           Rprintf("\nRSF:  *** ERROR *** ");
           Rprintf("\nRSF:  Factor level less than one (1):  %10.4f", _observation[_factorIndex[j]][i]);
-          Rprintf("\nRSF:  The application will now exit.\n");
-          exit(TRUE);
+          error("\nRSF:  The application will now exit.\n");
         }
       }
       if (_maxFactorLevel < _factorSize[j]) {
@@ -256,16 +216,14 @@ void initializeFactorArrays(char mode) {
           else {
             Rprintf("\nRSF:  *** ERROR *** ");
             Rprintf("\nRSF:  Factor level less than one (1):  %10.4f", _fobservation[_factorIndex[j]][i]);
-            Rprintf("\nRSF:  The application will now exit.\n");
-            exit(TRUE);
+            error("\nRSF:  The application will now exit.\n");
           }
         }
       }
       if (factorLevel > _factorSize[j]) {
         Rprintf("\nRSF:  *** ERROR *** ");
         Rprintf("\nRSF:  !GROW factor level greater than maximum GROW factor level:  %10d vs. %10d", factorLevel, _factorSize[j]);
-        Rprintf("\nRSF:  The application will now exit.\n");
-        exit(TRUE);
+        error("\nRSF:  The application will now exit.\n");
       }
     }
   }
@@ -285,7 +243,7 @@ char stackCompetingArrays(char mode) {
   char eventAnalysisFlag, consistencyFlag, overWriteFlag;
   char statusFlag;
   uint *eventCounter;
-  uint i, j, jgrow, n;
+  uint i, j, jgrow;
   _eventType = uivector(1, _observationSize);
   overWriteFlag = FALSE;
   if (mode == RSF_GROW) {
@@ -570,8 +528,7 @@ void stackFactorArrays(char mode) {
       Rprintf("\nRSF:  *** ERROR *** ");
       Rprintf("\nRSF:  Invalid predictor type:  [%10d] = %2s", p, _xType[p]);
       Rprintf("\nRSF:  Type must be 'C', 'I', or 'R'.");
-      Rprintf("\nRSF:  The application will now exit.\n");
-      exit(TRUE);
+      error("\nRSF:  The application will now exit.\n");
     }
   }
   _factorMap = uivector(1, _xSize);
@@ -628,8 +585,7 @@ char stackMissingSignatures(uint     obsSize,
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Attempt to allocate for missingness in its absence.");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
   }
   *p_recordIndex = uivector(1, recordSize);
   i = 0;
@@ -1203,8 +1159,7 @@ uint stackDefinedOutputObjects(char      mode,
         Rprintf("\nRSF:  *** ERROR *** ");
         Rprintf("\nRSF:  SEXP TREE output request inconsistent.");
         Rprintf("\nRSF:  Please Contact Technical Support.");
-        Rprintf("\nRSF:  The application will now exit.\n");
-        exit(TRUE);
+        error("\nRSF:  The application will now exit.\n");
       }
     }
     if ((_opt & OPT_MISS) && (_opt & OPT_OMIS)) {
@@ -1310,8 +1265,7 @@ uint stackDefinedOutputObjects(char      mode,
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Unknown case in switch encountered. ");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
     break;
   }
   performanceSize = adjEventSize * _forestSize;
@@ -1594,15 +1548,11 @@ void unstackDefinedOutputObjects(char      mode,
                                  Node    **root,
                                  double   *localSplitDepthPtr) {
   uint obsSize;
-  uint mRecordSize;
-  uint importanceSize;
   uint importanceMult;
   uint varUsedSize;
   uint adjEventSize;
   uint i, j;
   obsSize        = 0;  
-  mRecordSize    = 0;  
-  importanceSize = 0;  
   importanceMult = 0;  
   varUsedSize    = 0;  
   if (_eventTypeSize > 1) {
@@ -1614,7 +1564,6 @@ void unstackDefinedOutputObjects(char      mode,
   switch (mode) {
   case RSF_GROW:
     obsSize = _observationSize;
-    mRecordSize = _mRecordSize;
     if (_opt & OPT_VUSE) {
       if (_opt & (~OPT_VUSE) & OPT_VUSE_TYPE) {
         varUsedSize = _forestSize;
@@ -1626,39 +1575,31 @@ void unstackDefinedOutputObjects(char      mode,
     if (_opt & OPT_VIMP) {
       if (_opt & (~OPT_VIMP) & OPT_VIMP_JOIN) {
         importanceMult = 1;
-        importanceSize = adjEventSize;
       }
       else {
         importanceMult = _xSize;
-        importanceSize = adjEventSize * _xSize;
       }
     }
     break;
   case RSF_PRED:
     obsSize = _fobservationSize;
-    mRecordSize = _fmRecordSize;
     if (_opt & OPT_VIMP) {
       if (_opt & (~OPT_VIMP) & OPT_VIMP_JOIN) {
         importanceMult = 1;
-        importanceSize = adjEventSize;
       }
       else {
         importanceMult = _xSize;
-        importanceSize = adjEventSize * _xSize;
       }
     }
     break;
   case RSF_INTR:
     obsSize = _fobservationSize;
-    mRecordSize = _fmRecordSize;
     if (_opt & OPT_VIMP) {
       if (_opt & (~OPT_VIMP) & OPT_VIMP_JOIN) {
         importanceMult = 1;
-        importanceSize = adjEventSize;
       }
       else {
         importanceMult = _intrPredictorSize;
-        importanceSize = adjEventSize * _intrPredictorSize;
       }
     }
     break;
@@ -1666,8 +1607,7 @@ void unstackDefinedOutputObjects(char      mode,
     Rprintf("\nRSF:  *** ERROR *** ");
     Rprintf("\nRSF:  Unknown case in switch encountered. ");
     Rprintf("\nRSF:  Please Contact Technical Support.");
-    Rprintf("\nRSF:  The application will now exit.\n");
-    exit(TRUE);
+    error("\nRSF:  The application will now exit.\n");
     break;
   }
   if (_opt & OPT_FENS) {
